#******************************************************************************
#   MODcode_CSV.py 
#   Authors: Chrissy Rogers, Frank Freedman (San Jose State University)
#   Last Update: 9/20/2017
#
#   Code to read NASA hdf files from Dark Target, Deep Blue or MAIAC MODIS AOD products
#   and output a csv file of AOD and associated quantities around user specified locations 
#
#   See 'readme' file for further details
#
#   Provided to public free of charge, as is. 
#
#   This is NOT an official NASA product. 
#******************************************************************************

import time
import pandas as pd
from Python_Modules import Modis as modis 
import numpy as np
from pyhdf.SD import SD, SDC

Main_Directory="/home/freedman/CODES_TO_AQMD/CSV/ARSET/MAIAC/" #Directory of input files
Output_Directory='/home/freedman/CODES_TO_AQMD/CSV/ARSET/' #Directory of output file
Output_csv_file='test_Terra_M1.csv' #Enter name of output file
satellite='Terra'  # Terra or Aqua
Resolution=1 # 10km=10, 3km =3, 1km=1 enter integer not string
product='M' #DT=Dark_Target or DB=Deep_Blue or M=MAIAC 

#-----------------  #For Maiac Only (Enter None for others) ------------------
latlonfile=Main_Directory+"MAIACLatlon.h01v04.hdf" 

#----------------------------------------------------------------------------
# Do you want multiple locations y/n ?
#-----------------------------------------------------------------------------
Multiple='y' 

#***************************************************************************
#    If multiple = yes  (Multiple Cities) {Enter NONE if selecting One City}
#          Enter filepath and name of locations csv file, 
#      enter None if only selecting a single location then see below
#***************************************************************************
csv_file= "/home/freedman/CODES_TO_AQMD/CSV/ARSET/locations.csv"

#-------------------------------------------------------------------------------     
      # CSV file must include a header row
      # Columns should be labled: Location, Latitude, Longitude, Radius
      # Latitude, Longitude, and Radius should be in decimal degrees

#(*** FRF: Radius is actually for a bounding box, not a circle ***)

#------------------------------------------------------------------------------

#************************************************************************
# If multiple = No (Single City). { Enter NONE if selecting Multiple Cities}
#************************************************************************
user_lat=None#Location Latitude (-90.00 to 90.00)
user_lon=None #Location longitude (-180.00 to  180.00)
radius=None #radius of study area in decimal degrees (0.25)
Location=None #Location Name (e.g. 'SanJose')

#----------------------------------------------------------------------------
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#---------------------------------------------------------------------------
















#======================================================================
# Please do not alter code below
#=======================================================================
start = time.time() #needed to determine how long program takes to run.

if product=='M':
    hdflatlon=SD(latlonfile,SDC.READ)
        
    lat=hdflatlon.select('lat')[:,:]
    lon=hdflatlon.select('lon')[:,:]

if Multiple =='y' or Multiple=='Y':
    Locations=pd.read_csv(csv_file)
    df=pd.DataFrame({})
       
    for index, row in Locations.iterrows(): #iterate through locations
        Location,user_lat,user_lon,radius=row
#---------------------------------------------------------------------------------------------------------------------
        print '***************************************************************'
        print '                ',Location
        print '***************************************************************\n'
        if product=='M':
            valid_lat=(lat>=user_lat-radius)&(lat<=user_lat+radius)
            valid_lon=(lon>=user_lon-radius)&(lon<=user_lon+radius)
            valid_coord=np.where((valid_lat==True)&(valid_lon==True))
            d=modis.process_data(Main_Directory,Output_Directory,satellite,
                              Resolution,product,user_lat,user_lon,radius,
                              Location,valid_coord)
        else:
            d=modis.process_data(Main_Directory,Output_Directory,satellite,
                              Resolution,product,user_lat,user_lon,radius,
                              Location)

#--------------------------------------------------------------------------------------------------------------------                           radius,Location,latlonfile)["df"]
        df=pd.concat([df,d['df']])

else: #if single location
       df=modis.process_data(Main_Directory,Output_Directory,satellite,
          Resolution,product,user_lat,user_lon,radius,Location) 


df.to_csv(Output_Directory+Output_csv_file) #Create csv file
    
#======================================================================
#End Program
#======================================================================
timed_run = time.time() - start
print '***** Total time taken to run (s):', timed_run, '*****\n'
print '***** Total time taken to run (min):', timed_run/60, '*****\n'
