#******************************************************************************
#                       Written by: Chrissy Rogers                            
#                         P.I.: Frank Freedman                                
#                              July 2017                                      
#                            Python 2.7.6                                     
#******************************************************************************

import glob
from pyhdf.SD import SD, SDC
import numpy as np
import pandas as pd
import datetime as dt 

#*******************************************************************************
#*******************************************************************************
#                               
#                                NOTE
# 
#    lines 237,240,243,246,251 and/or 254 may need to be altered, 
#    depending on directory/subdirectory stucture of stored files
#
#******************************************************************************
#******************************************************************************

def file_details(FILE_NAME):#raw_input('Enter Your City Name in CamelCase: ')
    '''This function takes the filepath and splits it to keep only the hdf file
    name and extracts information given in the hdf naming convention prescribed
    by MODIS'''
    
    file_info=FILE_NAME.split('/')[-1] #keep file name only from file path
    
    sat_name= ''
    resolution=''
    
    if file_info[:5]=='MAIAC':
        sat=file_info[5]
        if sat=='T':
            sat_name='Terra'
        elif sat=='A':
            sat_name='Aqua'
        else: print "Unknown, different naming convention for file"
        resolution='1 km' #Maiac only has 1km resolution
        date=file_info[17:24]
        time=file_info[24:28]
    else:
        #First 3 Characters tell us which Satelite (Terra=MOD,Aqua=MYD)
        sat=file_info[:3] 
        if sat=="MOD": 
            sat_name='Terra'
        elif sat=="MYD":
            sat_name='Aqua'
        else: print "Unknown, different naming convention for file"
        res=file_info[6:8] #Characters 6 & 7 are resolution (L2=10 km,3K=3 km)
        if res=='L2':
            resolution= '10 km'
        elif res=='3K':
            resolution='3 km'
        else:print 'Unknown, different naming convention for file'
        date=file_info[10:17] #Characters 10-16 are year and Julian day
        time=file_info[18:22] #Characters 18-21 are hour and minute    
    #Convert Julian Date to Calender Date
    Date=dt.datetime.strptime(date,'%Y%j').strftime('%m/%d/%Y') 
    Time=dt.datetime.strptime(time,'%H%M').strftime('%H:%M')
    return sat,sat_name,resolution,Date,Time

# End of Function
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



def read_file(file_name,user_lat,user_lon,product,radius,valid_coord):
    '''Thes function reads in an .hdf file and outputs average aerosol data,
    QA flag counts, and number of cells used in average for a specified region
    
    If using the Maiac product please enter the maiac lat/lon file path
    
    latlonfile= separate file path for lat/lon for Maiac product it is 
                NOT needed for Deep Blue or Dark Target'''

    
    hdf = SD(file_name, SDC.READ)

#------------------------------------------------------------------------------- 
# Check for and Read in Maiac Lat/Lon file.  It is a separate file
#-------------------------------------------------------------------------------  
    if product=='M':
        scale_factor=0.001 #From Maiac readme file
#=======================================================================


#-------------------------------------------------------------------------------
#  Read in lat/lon for Deep Blue and Dark Target Files (inlcuded in hdf file)
#  Also read in QA flag data.   Maiac does not have QA flags
#-------------------------------------------------------------------------------       
    if product !='M':
        lat=hdf.select("Latitude")[:,:] #read all latitudes from hdf file
        lon=hdf.select("Longitude")[:,:] #read all longitudes from hdf file
        scale_factor = 0.0010000000474974513 #defined in hdf file metadata
          
        valid_lat=(lat>=user_lat-radius)&(lat<=user_lat+radius)
        valid_lon=(lon>=user_lon-radius)&(lon<=user_lon+radius)
        Valid_Coord=np.where((valid_lat==True)&(valid_lon==True))

        QA_input=hdf.select('Land_Ocean_Quality_Flag')[:,:].astype(np.float)
        QA_input[QA_input<=-1]=np.NaN 
        QA=QA_input[Valid_Coord[0],Valid_Coord[1]]
                
#=======================================================================


#-------------------------------------------------------------------------------
# Read in Aerosol data and locate bounding box
#-------------------------------------------------------------------------------

    if product=="DT": #Dark Target -  Read hdf file apply scale factor
        Opt_Depth_Land_2D=hdf.select('Corrected_Optical_Depth_Land')
        Opt_Depth_Land=Opt_Depth_Land_2D[:,:].astype(np.double)*scale_factor
        Opt_Depth_Land[Opt_Depth_Land<=-9]=np.NaN
        AOD=Opt_Depth_Land[:,Valid_Coord[0],Valid_Coord[1]]

    elif product=="DB": #Deep Blue - Read hdf file apply scale factor

        Opt_Depth_Land_2D=hdf.select('Deep_Blue_Aerosol_Optical_Depth_550_Land')
        Opt_Depth_Land=Opt_Depth_Land_2D[:,:].astype(np.double)*scale_factor
        Opt_Depth_Land[Opt_Depth_Land<=-9]=np.NaN
        AOD=Opt_Depth_Land[Valid_Coord[0],Valid_Coord[1]]

    elif product=="M":  #Maiac - Read hdf file apply scale factor
        AOD_47=hdf.select('Optical_Depth_047')[:,:]*scale_factor
        AOD_47[AOD_47<-1]=np.NaN
        AOD_47=AOD_47[valid_coord]
        
        AOD_55=hdf.select('Optical_Depth_055')[:,:]*scale_factor
        AOD_55[AOD_55<-1]=np.NaN
        AOD_55=AOD_55[valid_coord]
        AOD=np.stack((AOD_47,AOD_55),axis=0)

    else: print "Please choose a product \n"
#======================================================================             

#------------------------------------------------------------------------------
#   Check for all missing data
#-----------------------------------------------------------------------------
    if np.isnan(AOD).all()==False:
        print 'Valid data exists in current file {} \n'.format( 
                file_name.split('/')[-1]) #only prints to track file progression
        print '=============================================== \n'

    else:
        print "File {} contains no valid data for selected area".format(
                file_name.split('/')[-1])
        print "=============================================== \n"
        return #excludes file from processing
#*******************************************************************************

#------------------------------------------------------------------------------
#  Check for valid data in all bands and average bands together
#------------------------------------------------------------------------------
    if product=="DT": #Dark Target
        if len(AOD)!=3: # Checks to see if all three bands are present
            err="File {} does not contain all bands and was excluded from " \
                        "study".format(file_name.split('/')[-1])
            print err
            print '=============================================== \n'
            return #exclude file from processing
        else:
            
            AOD_averages=np.nanmean(AOD,1)
           
            
    if product=="M": #Maiac
        if len(AOD)!=2: # Checks to see if all three bands are present
            err="File {} does not contain all bands and was excluded from " \
                        "study".format(file_name.split('/')[-1])
            print err
            print '=============================================== \n'
            return #exclude file from processing
        else:
            AOD_averages=np.nanmean(AOD,1)

    elif product=="DB": #Deep Blue
        AOD_averages=np.nanmean(AOD)
 #*****************************************************************************
 
 
#-------------------------------------------------------------------------------
#  Count QA Flags  (NOTE:  Maiac Does not have QA flags)
#-------------------------------------------------------------------------------     
    if product != 'M': #if product is NOT maiac
       
        QA_count=np.count_nonzero(QA==0) # Bad Flags
        QA_count=np.append(QA_count,np.count_nonzero(QA==1)) #Marginal Flags
        QA_count=np.append(QA_count,np.count_nonzero(QA==2)) #Good Flags
        QA_count=np.append(QA_count,np.count_nonzero(QA==3)) #Very Good Flags
#******************************************************************************* 


#------------------------------------------------------------------------------
#  Number of cells used in Regional Average (Bounding Box) with values
#------------------------------------------------------------------------------      
    if product != 'DB':
        cells_per_band=np.count_nonzero(AOD[0]>-1)
    else:
        cells_per_band=np.count_nonzero(AOD>-1)
 
#******************************************************************************


#------------------------------------------------------------------------------
# Return Data  (NOTE: Maiac does not have QA flags)
#------------------------------------------------------------------------------
    if product !='M':
        return AOD_averages.tolist(), QA_count.tolist(),cells_per_band
    else:
        return AOD_averages.tolist(),0,cells_per_band

# End of Function
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@   


def process_data(Main_Directory,Output_Directory,satellite,Resolution,
           product,user_lat,user_lon,radius,Location,valid_coord=None):
    '''This function processes the Modis data set to find files with
    requested satellite, resolution, lat and lon, and surronding radius and 
    outputs at pandas dataframe
    latlonfile= separate file path for lat/lon for Maiac product it is 
                NOT needed for Deep Blue or Dark Target'''


#------------------------------------------------------------------------------
#  Find all files to read in.  (Maiac only has 1km Resolution)
#------------------------------------------------------------------------------
    if product !='M': #if product is NOT Maiac
        if satellite=="Terra" and Resolution==10: 
            files=glob.glob(Main_Directory + '*/MOD04_L*.hdf')
            files.sort()
        elif satellite=="Terra" and Resolution==3:
            files=glob.glob(Main_Directory  +'*/MOD04_3*.hdf') 
            files.sort()
        elif satellite=="Aqua" and Resolution==10:
            files=glob.glob(Main_Directory + '*/MYD04_L*.hdf')
            files.sort()
        elif satellite=="Aqua" and Resolution==3:
            files=glob.glob(Main_Directory + '*/MYD04_3*.hdf')
            files.sort()
        else: print 'Please enter Satellite and Resolution to choose files'
    else: #if product is Maiac
        if satellite=="Terra":
            files=glob.glob(Main_Directory  +'*/MAIACT*.hdf')#     May need to alter search path
            files.sort()
        elif satellite=="Aqua":
            files=glob.glob(Main_Directory + '*/MAIACA*.hdf')#  May need to alter search path
            files.sort()        
#=======================================================================

#-------------------------------------------------------------------------------
#  Create empty DataFrames to hold data
#-------------------------------------------------------------------------------
    if product=='DT':
        df=pd.DataFrame({'Date':['01/01/0000'],'Time':['12:00'],
                         '0.47_microns':[0.00],'0.55_microns':[0.00],
                         '0.66_microns':[0.00],
                         'Number_QA_Bad':[0],'Number_QA_Marginal':[0],
                         'Number_QA_Good':[0],'Number_QA_VeryGood':[0],
                         'Number_Valid_Cells':[0]})
        order=['Time','Location','0.47_microns','0.55_microns',
                '0.66_microns','Number_QA_Bad',
                'Number_QA_Marginal','Number_QA_Good','Number_QA_VeryGood',
                'Number_Valid_Cells','Satellite','Product']
        
    elif product=='DB':
        df=pd.DataFrame({'Date':['01/01/0000'],'Time':['12:00'],'AOD':[0.00],
                         'Number_QA_Bad':[0],'Number_QA_Marginal':[0],
                         'Number_QA_Good':[0],'Number_QA_VeryGood':[0],
                         'Number_Valid_Cells':[0]})
        order=['Time','Location','AOD','Number_QA_Bad',
               'Number_QA_Marginal','Number_QA_Good','Number_QA_VeryGood',
               'Number_Valid_Cells','Satellite', 'Product']
    elif product=='M':
        df=pd.DataFrame({'Date':['01/01/0000'],'Time':['12:00'],
                         '0.47_microns':[0.00],'0.55_microns':[0.00],
                         'Number_Valid_Cells':[0]}) 
        order=['Time','Location','0.47_microns','0.55_microns',
               'Number_Valid_Cells', 'Satellite', 'Product']         
       
#=======================================================================


#-------------------------------------------------------------------------------
#  Read each file
#-------------------------------------------------------------------------------
    for file_name in files[:]:                                        ##############################################
        if product=='M':
            try:
                AOD_averages, QA_count,cells_per_band=read_file(file_name,
                                                   user_lat,user_lon,product,
                                                   radius,valid_coord)
                sat,sat_name,resolution,date,time=file_details(file_name)
        
            except:
               continue #if error move to next file
        else:
            try:
                AOD_averages, QA_count,cells_per_band=read_file(file_name,
                                                   user_lat,user_lon,product,
                                                   radius,valid_coord=None)
                sat,sat_name,resolution,date,time=file_details(file_name)
                
            except:
                continue #if error move to next file
                  

#=======================================================================


#-------------------------------------------------------------------------------
# Store data in DataFrame
#-------------------------------------------------------------------------------            

        values=[date]
        values.append(time)
        if product=='DB':
            values.append(AOD_averages)
        else:
            values.extend(AOD_averages)
        values.append(cells_per_band)
              
        if product !='M':
            values.extend(QA_count)
           
        if product=='DT': #Dark Target
            df1=pd.DataFrame(data=[values],
                  columns=['Date','Time','0.47_microns','0.55_microns',
                           '0.66_microns','Number_Valid_Cells',
                           'Number_QA_Bad','Number_QA_Marginal','Number_QA_Good',
                           'Number_QA_VeryGood'])
           
        elif product=='DB': #Deep Blue
            df1=pd.DataFrame(data=[values],
                  columns=['Date','Time','AOD','Number_Valid_Cells',
                          'Number_QA_Bad','Number_QA_Marginal','Number_QA_Good',
                          'Number_QA_VeryGood'])
        elif product=='M': #Maiac
            df1=pd.DataFrame(data=[values],
                  columns=['Date','Time','0.47_microns','0.55_microns',
                           'Number_Valid_Cells'])
                  
        df=pd.concat([df,df1])
        
    df=df.set_index('Date')
    df=df.drop('01/01/0000') #get rid of initializaion date
    df.index=pd.to_datetime(df.index)
    
#=======================================================================


#-------------------------------------------------------------------------------  
# Weighted Average multiple files on same day
#-------------------------------------------------------------------------------
    if product=='DT': #Dark Target
        try:
            band=df.resample('D').apply(lambda x:pd.Series(np.average(x[
              ["0.47_microns","0.55_microns","0.66_microns"]],
              weights=x['Number_Valid_Cells'],axis=0),['0.47_microns',
              '0.55_microns','0.66_microns']))
            
        except:
            band=df.resample('D').apply(lambda x:pd.Series(np.mean(x[
              ["0.47_microns","0.55_microns","0.66_microns"]]),      
              ['0.47_microns','0.55_microns','0.66_microns']))
                  
    
        band.drop(['Time','Number_Valid_Cells','Number_QA_Bad',
                  'Number_QA_Marginal','Number_QA_Good','Number_QA_VeryGood'],
                   axis=1,inplace=True)
                   
                        
        T=df[['Time','Number_Valid_Cells']].copy()
        T=T.sort_values('Number_Valid_Cells').groupby(
           T.sort_values('Number_Valid_Cells').index).agg(
           {'Number_Valid_Cells':'sum','Time':'last'})
        T.drop('Number_Valid_Cells',axis=1,inplace=True)
               
        
        df.drop(['Time','0.47_microns','0.55_microns','0.66_microns'
                 ],axis=1,inplace=True)
        df=df.resample('D').sum().fillna(-999).astype(int)
       
    elif product=='DB': #Deep Blue
        try:
            band=df.resample('D').apply(lambda x:pd.Series(np.average(x[
              ["AOD"]],
              weights=x['Number_Valid_Cells'],axis=0),['AOD']))
            band.drop(['Time','Number_Valid_Cells','Number_QA_Bad',
                'Number_QA_Marginal','Number_QA_Good','Number_QA_VeryGood'],
                 axis=1,inplace=True)
           
        except:
            band=df.resample('D').agg({'AOD':np.mean})
    
        
        T=df[['Time','Number_Valid_Cells']].copy()
        T=T.sort_values('Number_Valid_Cells').groupby(
           T.sort_values('Number_Valid_Cells').index).agg(
           {'Number_Valid_Cells':'sum','Time':'last'})
        T.drop('Number_Valid_Cells',1,inplace=True)
       
        df.drop(['Time','AOD'],axis=1,inplace=True)
        df=df.resample('D').sum().fillna(-999).astype(int)
        
    elif product=='M': #Maiac
        try:
            band=df.resample('D').apply(lambda x:pd.Series(
               np.average(x[["0.47_microns", "0.55_microns"]],
                  weights=x['Number_Valid_Cells'],axis=0),['0.47_microns',
                         '0.55_microns']))
            
        except:
            band=df.resample('D').apply(lambda x:pd.Series(np.mean(x[
             ["0.47_microns","0.55_microns"]]),      
             ['0.47_microns','0.55_microns']))
              
        band.drop(['Time','Number_Valid_Cells'],axis=1,inplace=True)
        
        T=df[['Time','Number_Valid_Cells']].copy()
        T=T.sort_values('Number_Valid_Cells').groupby(
           T.sort_values('Number_Valid_Cells').index).agg(
           {'Number_Valid_Cells':'sum','Time':'last'})
        T.drop('Number_Valid_Cells',1,inplace=True)
       
        df.drop(['Time','0.47_microns','0.55_microns'],
                 axis=1,inplace=True)
        df=df.resample('D').sum().fillna(-999).astype(int)
#=======================================================================
 

#-------------------------------------------------------------------------------
#  Concatenate files and add Location, Satellite, Product, and Resolution
#   to the DataFrame
#-------------------------------------------------------------------------------    
    
    df2=pd.concat([df,T],axis=1)
    df2=pd.concat([band.fillna(-999),df2],axis=1)    
    df2['Location']=Location
    df2['Satellite']=satellite
    df2['Product']=product+str(Resolution)+'km'

#=======================================================================


#-------------------------------------------------------------------------------                
#  Round to correct number of decimal places
#-------------------------------------------------------------------------------    
    if product=='DT':
        df2=df2.round({'0.47_microns':4,'0.55_microns':4,'0.66_microns':4})
        band=band.round({'0.47_microns':4,'0.55_microns':4,'0.66_microns':4})
      
    elif product=='DB':
        df2=df2.round({'AOD':4})
        df2=df2.round({'AOD':4})

    elif product=='M':
        df2=df2.round({'0.47_microns':4,'0.55_microns':4})
        band=band.round({'0.47_microns':4,'0.55_microns':4})
 #======================================================================
 

#-------------------------------------------------------------------------------
#  Store time as Ordinal for future NetCDF file type functionality
#-------------------------------------------------------------------------------   
    Hour=T.reset_index(level=0)
    Hour['Time']=pd.to_datetime(Hour['Time'],format='%H:%M').dt.time
    Hour=Hour[['Date','Time']].apply(lambda x: 
                                      dt.datetime.combine(*list(x)),axis=1)
    Hour=Hour.apply(lambda x:x.toordinal()).values

#=======================================================================    


#-------------------------------------------------------------------------------
#Create Dictonary to store all DataFrames
#-------------------------------------------------------------------------------
    df[df<-1]=np.nan
    df2=df2[pd.notnull(df2['Time'])]
    df2=df2.reindex_axis(order,axis=1)
        
    d={'df':df2,'df_QA_Cell':df,'band':band,'Time':Hour}
#=======================================================================                    
    
    return d
#  End of Function
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


#======================================================================
#Function below is for future NetCDF file type functionality
#======================================================================
def Loc_Concat(dictonary,key,column,Location):
    '''To be used in loop to concatenate locations for each variable for 
    netcdf file output. This outputs a pandas dataFrame.
    index=index variable in for loop
    dictonary=dictonary name data is stored in
    key=key of data stored in dictornary (string)
    column=column name of key which is a dataframe (string)'''

    variable=pd.DataFrame(dictonary[key][column])                               
    variable.columns=[Location]
    variable.set_index='Date'
    
    return variable
    
